package dev.inmo.postssystem.client

import dev.inmo.micro_utils.fsm.common.managers.DefaultStatesManagerRepo
import dev.inmo.postssystem.features.common.common.ui.fsm.UIFSMState
import kotlinx.browser.window
import org.w3c.dom.*
import org.w3c.dom.url.URL

private fun History.refreshHistory(
    states: Iterable<UIFSMState>,
    fillers: List<UIFSMStateSearchParamsHandler>
) {
    val currentUrl = window.location.pathname
    val currentParams = (URL(window.location.href)).searchParams
    val params = states.flatMap<UIFSMState, Pair<String, String>> { state ->
        fillers.flatMap {
            it.takeParams(state, currentParams)
        }
    }
    pushState(
        null,
        "Posts System",
        "$currentUrl${params.joinToString("&", "?") { "${it.first}=${it.second}" }}"
    )
}

private fun takeStates(initialState: UIFSMState, fillers: List<UIFSMStateSearchParamsHandler>): List<UIFSMState> {
    val params = (URL(window.location.href)).searchParams
    val additionalStates = fillers.mapNotNull {
        it.takeState(params)
    }

    return additionalStates + listOfNotNull(
        if (additionalStates.isEmpty()) {
            initialState
        } else {
            null
        }
    )
}



class JSUIFSMStatesRepo(
    private val history: History,
    private val initialState: UIFSMState,
    private val fillers: List<UIFSMStateSearchParamsHandler>
) : DefaultStatesManagerRepo<UIFSMState> {
    private val statesMap = mutableMapOf<String, UIFSMState>()

    init {
        val states = takeStates(initialState, fillers)
        states.forEach {
            statesMap[it.context] = it
        }
    }

    override suspend fun getContextState(context: Any): UIFSMState? {
        return when (context) {
            is String -> statesMap[context]
            else -> null
        }
    }

    override suspend fun contains(context: Any): Boolean = when (context) {
        is String -> statesMap.contains(context)
        else -> super.contains(context)
    }

    override suspend fun getStates(): List<UIFSMState> = statesMap.values.toList()

    override suspend fun removeState(state: UIFSMState) {
        statesMap.remove((state.context as? String) ?: return)
        history.refreshHistory(statesMap.values, fillers)
    }

    override suspend fun set(state: UIFSMState) {
        console.log(state)
        statesMap[state.context] = state
        history.refreshHistory(statesMap.values, fillers)
    }
}
