package dev.inmo.postssystem.features.content.binary.client

import androidx.compose.runtime.*
import dev.inmo.jsuikit.elements.*
import dev.inmo.jsuikit.modifiers.UIKitWidth
import dev.inmo.micro_utils.common.selectFile
import dev.inmo.micro_utils.mime_types.KnownMimeTypes
import dev.inmo.postssystem.features.common.common.*
import dev.inmo.postssystem.features.content.client.ContentClientProvider
import dev.inmo.postssystem.features.content.common.*
import org.jetbrains.compose.web.dom.Img
import org.jetbrains.compose.web.dom.Text
import org.koin.core.module.Module

object LoadingClientModule : ModuleLoader {
    init {
        AdditionalModules.Default.addModule(this)
    }

    override fun Module.load() {
        singleWithRandomQualifier<ContentClientProvider> {
            BinaryContentClientProvider
        }
    }
}

private val loadingClientModuleForLoadingAtRuntime = LoadingClientModule

object BinaryContentClientProvider : ContentClientProvider {
    override fun contentTypeNameForUser(): String = "File"

    @Composable
    override fun renderNewInstance(state: MutableState<Content?>) {
        console.log(state.value)
        val value = (state.value as? BinaryContent)
        if (value == null) {
            selectFile {
                state.value = it.binaryContent()
            }
        }
        DefaultButton(value ?.filename ?.name ?: "Select file", UIKitWidth.Expand) {
            selectFile {
                state.value = it.binaryContent()
            }
        }
    }

    @Composable
    override fun renderPreview(content: Content): Boolean {
        if (content is BinaryContent) {
            Tile {
                Card(
                    header = {
                        CardTitle {
                            Text(content.filename.name)
                        }
                    }
                ) {}
            }

            return true
        }

        return false
    }
}
